module Comp where

import Data.List(isSuffixOf)
import System.Environment(getProgName, getArgs)   
import Control.Exception(catch)


import Absy
import Parser

comp :: Int-> Cmd-> String 
comp i FWD = prt i "++ptr;"
comp i BWD = prt i "--ptr;"
comp i INC = prt i "++(*ptr);"
comp i DEC = prt i "--(*ptr);"
comp i OUT = prt i "putchar(*ptr);"
comp i INP = prt i "*ptr= getchar();"
comp i (WHL b) = 
  prt i "while (*ptr) {" ++ concatMap (comp (i+1)) b ++ prt i "}"

prt :: Int-> String-> String
prt i c = spaces (2*i) ++ c ++ "\n"

spaces :: Int-> String
spaces i = take i (repeat ' ')

header fp prog = 
  "/* Source: "++ fp++ " */\n"++
  "#include <stdio.h>\n\n"++
  "char array[32768] = {0}; char *ptr= array;\n"++
  "int main(int argc, char *argv) {\n"++
  prog++
  "}\n"

compile :: FilePath-> IO ()
compile fp = do
  pr <- parseFile fp
  case pr of 
    Right cs -> do
      let out= targetFile fp 
      writeFile out $ header fp (concatMap (comp 1) cs)
      putStrLn $ "Output written to "++ out
    Left e -> putStrLn $ "*** ERROR: "++ e


targetFile :: FilePath-> FilePath 
targetFile srcnm
  | ".b" `isSuffixOf` srcnm = take (length srcnm - 2) srcnm ++ ".c"
  | ".bf" `isSuffixOf` srcnm = take (length srcnm - 3) srcnm ++ ".c"
  | otherwise = srcnm ++ ".c" 

main :: IO ()
main = do
  putStrLn "Welcome to Brainf***"
  p <- getProgName
  args <- getArgs
  case args of 
    [f] -> compile f
    _   -> do putStrLn $ "*** ERROR: Usage "++ show p++ " <filename>"
