module Parser(parser, parseFile) where

import Control.Exception
import Data.Char(isSpace)

import Absy

comments :: String-> String 
comments = takeWhile ('#' /=)

lexer :: String-> String 
lexer = concatMap (filter (not. isSpace) . comments) . lines

cons1 :: a-> Either c ([a], b)-> Either c ([a], b)
cons1 x (Right (xs, ys)) = Right (x:xs, ys)
cons1 _  l = l

-- Returns either unparseable rest, or sequence of cmds and not yet parsed rest
parseCmds :: String-> Either String ([Cmd], String)
parseCmds [] = Right ([], "")
parseCmds (c:cs) = case c of 
  '<' -> cons1 BWD $ parseCmds cs
  '>' -> cons1 FWD $ parseCmds cs
  '+' -> cons1 INC $ parseCmds cs
  '-' -> cons1 DEC $ parseCmds cs
  '.' -> cons1 OUT $ parseCmds cs
  ',' -> cons1 INP $ parseCmds cs
  '[' -> case parseCmds cs of 
           Right (b, ']':r) -> cons1 (WHL b) $ parseCmds r
           Right (b, []) -> Left $ "Missing closing ']' in " ++ cs
           err -> err
  ']' -> Right ([], c:cs) 
  _   -> parseCmds cs -- We just skip anything else
                      -- Alternative is to throw an error: Left $ "Unexpected command `"++ c : "`" 

parser :: String-> Either String [Cmd]
parser str = case parseCmds (lexer str) of 
   Right (cs, []) -> Right cs
   Right (_,  cs) -> Left $ "Unexpected unparsed cmds `"++ cs ++ "`" 
        -- Can't actually happen b/c we skip everything we cinnae parse
   Left e -> Left e

parseFile :: FilePath-> IO (Either String [Cmd])
parseFile path = (do
  f<- readFile path
  return $ parser f 
  ) `catch` (\e -> return $ Left $ show (e:: IOException))
